@@ -38,6 +38,10 @@ mod sealed {
 
 const TCP_KEEPALIVE: Duration = Duration::from_secs(60);
 
+// ClickHouse uses 3s by default.
+// See https://github.com/ClickHouse/ClickHouse/blob/368cb74b4d222dc5472a7f2177f6bb154ebae07a/programs/server/config.xml#L201
+const POOL_IDLE_TIMEOUT: Duration = Duration::from_secs(2);
+
 #[derive(Clone)]
 pub struct Client {
     client: hyper::Client<HttpConnector>,
@@ -57,8 +61,12 @@ impl Default for Client {
         // TODO: make configurable in `Client::builder()`.
         connector.set_keepalive(Some(TCP_KEEPALIVE));
 
+        let client = hyper::Client::builder()
+            .pool_idle_timeout(POOL_IDLE_TIMEOUT)
+            .build(connector);
+
         Self {
-            client: hyper::Client::builder().build(connector),
+            client,
             url: String::new(),
             database: None,
             user: None,
